// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

// UE includes
#include "CoreMinimal.h"
#include "GameFramework/CharacterMovementComponent.h"

#include "GSCharacterMovementComponent.generated.h"

/**
 *	Subclass of CharacterMovementComponent which allows for a custom gravity direction to be applied to the character while falling
 */
UCLASS()
class GRAVITYSTOMPGAME_API UGSCharacterMovementComponent : public UCharacterMovementComponent
{
	GENERATED_BODY()

public:
	UGSCharacterMovementComponent();

	UFUNCTION(BlueprintCallable)
	void SetCharacterUpDirection(FVector NewUpDirection);

	// UCharacterMovementComponent interface
	virtual void PhysFalling(float deltaTime, int32 Iterations) override;

	virtual void ComputeFloorDist(const FVector& CapsuleLocation, float LineDistance, float SweepDistance, FFindFloorResult& OutFloorResult, float SweepRadius, const FHitResult* DownwardSweepResult) const override;

	virtual bool IsValidLandingSpot(const FVector& CapsuleLocation, const FHitResult& Hit) const override;

	virtual bool IsWalkable(const FHitResult& Hit) const override;
	// End UCharacterMovementComponent interface

private:
	FVector CharacterUpDirection = FVector::UpVector;
};
